@@ -288,6 +288,7 @@ fn concat<
     let p = cursor.len();
     let rc_level = cursor.first_rc_len();
     let curs = cursor.current_mut();
+
     if p == p0 {
         // Here, p == p0, meaning that we're deleting at an internal
         // node. If that's the case, k0v0 is `Some`, hence we can
@@ -501,15 +502,21 @@ fn handle_merge<
                 last_op.skip_first = false;
 
                 // If this split isn't at the root, and the page
-                // modified in the last step is the one on the left of
-                // the entry, move back one step before inserting the
-                // split key/value.
+                // modified in the last step is the one on the right
+                // of the current entry, move right one step before
+                // inserting the split key/value.
                 //
                 // (remember that we popped the stack upon entering
                 // this function).
+                //
+                // We need to do this because the split key/value is
+                // inserted immediately *before* the current cursor
+                // position, which is incorrect if the split key/value
+                // comes from the right-hand side of the current
+                // cursor position.
                 if cursor.len() > 0 && !mod_is_left {
                     let curs = cursor.current_mut();
-                    P::move_prev(&mut curs.cursor);
+                    P::move_next(&mut curs.cursor);
                 }
                 // If the split key is the replacement element, we have
                 // already increased its counter when we deleted it from
